#
# This file is part of POE-Component-SSLify
#
# This software is copyright (c) 2014 by Apocalypse.
#
# This is free software; you can redistribute it and/or modify it under
# the same terms as the Perl 5 programming language system itself.
#
use strict; use warnings;
# This Build.PL for POE-Component-SSLify was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.008.
use strict;
use warnings;

use 5.006;
use Module::Build::Tiny 0.039;
Build_PL();
